@@ -173,56 +173,50 @@ static const char *res2str(TermKeyResult res)
 }
 #endif
 
-/* We might expose this as public API one day, when the ideas are finalised.
- * As yet it isn't public, so keep it static
- */
-static TermKey *termkey_new_full(int fd, int flags, size_t buffsize, int waittime)
+static TermKey *termkey_alloc(void)
 {
-  TermKey *tk = malloc(sizeof(*tk));
+  TermKey *tk = malloc(sizeof(TermKey));
   if(!tk)
     return NULL;
 
-  if(!(flags & (TERMKEY_FLAG_RAW|TERMKEY_FLAG_UTF8))) {
-    int locale_is_utf8 = 0;
-    char *e;
+  /* Default all the object fields but don't allocate anything */
 
-    if((e = getenv("LANG")) && strstr(e, "UTF-8"))
-      locale_is_utf8 = 1;
-
-    if(!locale_is_utf8 && (e = getenv("LC_MESSAGES")) && strstr(e, "UTF-8"))
-      locale_is_utf8 = 1;
-
-    if(!locale_is_utf8 && (e = getenv("LC_ALL")) && strstr(e, "UTF-8"))
-      locale_is_utf8 = 1;
-
-    if(locale_is_utf8)
-      flags |= TERMKEY_FLAG_UTF8;
-    else
-      flags |= TERMKEY_FLAG_RAW;
-  }
-
-  tk->fd    = fd;
-  tk->flags = flags;
+  tk->fd         = -1;
+  tk->flags      = 0;
   tk->canonflags = 0;
 
-  if(flags & TERMKEY_FLAG_SPACESYMBOL)
-    tk->canonflags |= TERMKEY_CANON_SPACESYMBOL;
-
-  tk->buffer = malloc(buffsize);
-  if(!tk->buffer)
-    goto abort_free_tk;
-
+  tk->buffer    = NULL;
   tk->buffstart = 0;
   tk->buffcount = 0;
-  tk->buffsize  = buffsize;
+  tk->buffsize  = 256; /* bytes */
 
   tk->restore_termios_valid = 0;
 
-  tk->waittime = waittime;
+  tk->waittime = 50; /* msec */
 
   tk->is_closed = 0;
 
   tk->nkeynames = 64;
+  tk->keynames  = NULL;
+
+  for(int i = 0; i < 32; i++)
+    tk->c0[i].sym = TERMKEY_SYM_NONE;
+
+  tk->drivers = NULL;
+
+  tk->method.emit_codepoint = &emit_codepoint;
+  tk->method.peekkey_simple = &peekkey_simple;
+  tk->method.peekkey_mouse  = &peekkey_mouse;
+
+  return tk;
+}
+
+static int termkey_init(TermKey *tk, const char *term)
+{
+  tk->buffer = malloc(tk->buffsize);
+  if(!tk->buffer)
+    return 0;
+
   tk->keynames = malloc(sizeof(tk->keynames[0]) * tk->nkeynames);
   if(!tk->keynames)
     goto abort_free_buffer;
@@ -231,13 +225,6 @@ static TermKey *termkey_new_full(int fd, int flags, size_t buffsize, int waittim
   for(i = 0; i < tk->nkeynames; i++)
     tk->keynames[i] = NULL;
 
-  for(i = 0; i < 32; i++)
-    tk->c0[i].sym = TERMKEY_SYM_NONE;
-
-  tk->method.emit_codepoint = &emit_codepoint;
-  tk->method.peekkey_simple = &peekkey_simple;
-  tk->method.peekkey_mouse  = &peekkey_mouse;
-
   for(i = 0; keynames[i].name; i++)
     if(termkey_register_keyname(tk, keynames[i].sym, keynames[i].name) == -1)
       goto abort_free_keynames;
@@ -247,8 +234,6 @@ static TermKey *termkey_new_full(int fd, int flags, size_t buffsize, int waittim
   register_c0(tk, TERMKEY_SYM_ENTER,     0x0d, NULL);
   register_c0(tk, TERMKEY_SYM_ESCAPE,    0x1b, NULL);
 
-  const char *term = getenv("TERM");
-
   struct TermKeyDriverNode *tail = NULL;
 
   for(i = 0; drivers[i]; i++) {
@@ -281,13 +266,13 @@ static TermKey *termkey_new_full(int fd, int flags, size_t buffsize, int waittim
   }
 
   if(!tk->drivers) {
-    fprintf(stderr, "Unable to find a terminal driver\n");
+    errno = ENOENT;
     goto abort_free_keynames;
   }
 
-  if(fd != -1 && !(flags & TERMKEY_FLAG_NOTERMIOS)) {
+  if(tk->fd != -1 && !(tk->flags & TERMKEY_FLAG_NOTERMIOS)) {
     struct termios termios;
-    if(tcgetattr(fd, &termios) == 0) {
+    if(tcgetattr(tk->fd, &termios) == 0) {
       tk->restore_termios = termios;
       tk->restore_termios_valid = 1;
 
@@ -296,7 +281,7 @@ static TermKey *termkey_new_full(int fd, int flags, size_t buffsize, int waittim
       termios.c_cc[VMIN] = 1;
       termios.c_cc[VTIME] = 0;
 
-      if(flags & TERMKEY_FLAG_CTRLC)
+      if(tk->flags & TERMKEY_FLAG_CTRLC)
         /* want no signal keys at all, so just disable ISIG */
         termios.c_lflag &= ~ISIG;
       else {
@@ -312,7 +297,7 @@ static TermKey *termkey_new_full(int fd, int flags, size_t buffsize, int waittim
 #ifdef DEBUG
       fprintf(stderr, "Setting termios(3) flags\n");
 #endif
-      tcsetattr(fd, TCSANOW, &termios);
+      tcsetattr(tk->fd, TCSANOW, &termios);
     }
   }
 
@@ -326,7 +311,7 @@ static TermKey *termkey_new_full(int fd, int flags, size_t buffsize, int waittim
   fprintf(stderr, "Drivers started; termkey instance %p is ready\n", tk);
 #endif
 
-  return tk;
+  return 1;
 
 abort_free_drivers:
   for(p = tk->drivers; p; ) {
@@ -342,15 +327,64 @@ abort_free_keynames:
 abort_free_buffer:
   free(tk->buffer);
 
-abort_free_tk:
-  free(tk);
-
-  return NULL;
+  return 0;
 }
 
 TermKey *termkey_new(int fd, int flags)
 {
-  return termkey_new_full(fd, flags, 256, 50);
+  TermKey *tk = termkey_alloc();
+  if(!tk)
+    return NULL;
+
+  tk->fd = fd;
+
+  if(!(flags & (TERMKEY_FLAG_RAW|TERMKEY_FLAG_UTF8))) {
+    int locale_is_utf8 = 0;
+    char *e;
+
+    if((e = getenv("LANG")) && strstr(e, "UTF-8"))
+      locale_is_utf8 = 1;
+
+    if(!locale_is_utf8 && (e = getenv("LC_MESSAGES")) && strstr(e, "UTF-8"))
+      locale_is_utf8 = 1;
+
+    if(!locale_is_utf8 && (e = getenv("LC_ALL")) && strstr(e, "UTF-8"))
+      locale_is_utf8 = 1;
+
+    if(locale_is_utf8)
+      flags |= TERMKEY_FLAG_UTF8;
+    else
+      flags |= TERMKEY_FLAG_RAW;
+  }
+
+  termkey_set_flags(tk, flags);
+
+  const char *term = getenv("TERM");
+
+  if(!termkey_init(tk, term)) {
+    free(tk);
+    return NULL;
+  }
+
+  return tk;
+}
+
+TermKey *termkey_new_abstract(const char *term, int flags)
+{
+  TermKey *tk = termkey_alloc();
+  if(!tk)
+    return NULL;
+
+  tk->fd = -1;
+
+  termkey_set_flags(tk, flags);
+
+  if(!termkey_init(tk, term)) {
+    free(tk);
+    return NULL;
+  }
+
+  return tk;
 }
 
 void termkey_free(TermKey *tk)
@@ -427,6 +461,23 @@ void termkey_set_canonflags(TermKey *tk, int flags)
     tk->flags &= ~TERMKEY_FLAG_SPACESYMBOL;
 }
 
+size_t termkey_get_buffer_size(TermKey *tk)
+{
+  return tk->buffsize;
+}
+
+int termkey_set_buffer_size(TermKey *tk, size_t size)
+{
+  unsigned char *buffer = realloc(tk->buffer, size);
+  if(!buffer)
+    return 0;
+
+  tk->buffer = buffer;
+  tk->buffsize = size;
+
+  return 1;
+}
+
 size_t termkey_get_buffer_remaining(TermKey *tk)
 {
   /* Return the total number of free bytes in the buffer, because that's what
@@ -1080,7 +1131,7 @@ static TermKeySym register_c0(TermKey *tk, TermKeySym sym, unsigned char ctrl, c
 static TermKeySym register_c0_full(TermKey *tk, TermKeySym sym, int modifier_set, int modifier_mask, unsigned char ctrl, const char *name)
 {
   if(ctrl >= 0x20) {
-    fprintf(stderr, "Cannot register C0 key at ctrl 0x%02x - out of bounds\n", ctrl);
+    errno = EINVAL;
     return -1;
   }
 
@@ -1103,7 +1154,7 @@ size_t termkey_snprint_key(TermKey *tk, char *buffer, size_t len, TermKeyKey *ke
   return termkey_strfkey(tk, buffer, len, key, format);
 }
 
-struct modnames {
+static struct modnames {
   const char *shift, *alt, *ctrl;
 }
 modnames[] = {
